% This is a simple test file to show how a DLL can be accessed directly from 
% Matlab, without using a MEX-file.
% This works in Matlab 6.5 and higher.
% M.Plat
% TiePie engineering

if ~exist( 'sDLL' )  
  sDLL = 'HS805';
  disp( [ 'DLL ' sDLL '.dll will be used. To use another DLL, set ''sDLL'' to another filename before executing this script.' ] );
end

% Load library if that is not already done:
if ~libisloaded( sDLL )
  [ notfound , warnings ] = loadlibrary( sDLL , 'TiePie.h' );
  pause( 0.2 );
  if ~libisloaded( sDLL )
    error( [ 'Could not load library ' , sDLL ] )
  end  
end

% Note: You can execute 'libfunctions( sDLL )' to see a list of the functions loaded from the DLL.

if ~exist( 'Const' )
  % Load all #define constants in TiePie.h into the structure 'Const':  
  Const = GetConstDefines( 'TiePie.h' );
  % Constants like Const.ltsCh1 can now be used.
end

% Initialize oscilliscope:
try
  Result = calllib( sDLL , 'InitInstrument' , 0 );
end  
if Result
  error( [ 'Could not initialize ' , sDLL ] );
end

% Get channel count:
[ Result , nChannelCount ] = calllib( sDLL , 'GetNrChannels' , 0 );

% Enable all channels:
try
  for i = 1:nChannelCount;    
    Result = calllib( sDLL , 'SetChEnabled' , i , 1 );  
  end
catch
  % If SetChEnabled is not supported by the DLL, use SetMeasureMode:
  Result = calllib( sDLL , 'SetMeasureMode' , 255 );  
end

for i = 1:nChannelCount;
  % Set channel range in Volts:
  calllib( sDLL , 'SetSensitivity' , i , 8 );
end

calllib( sDLL , 'SetRecordLength', 5000 );
% Get the actual record length:
nRecordLength = calllib( sDLL , 'GetRecordLength' );

% Set the post sample count to same number as record length, to record all
% samples after the trigger:
Result = calllib( sDLL , 'SetPostSamples' , nRecordLength );

% Set the resolution, if other than default resolution is needed:
% Result = calllib( sDLL , 'SetResolution' , 16 );

% Set and get the sample frequency:
[ Result rSampleFrequency ] = calllib( sDLL , 'SetSampleFrequencyF', 10e6 );

% Set trigger time out to 100 ms to make sure a measurement is also done when there is no trigger.
% Note: trigger time out does only work with the HS805 and the Multi Channel DLL.
calllib( sDLL , 'SetTriggerTimeOut' , 100 );

% Set channel 1 as trigger source:
calllib( sDLL , 'SetTriggerSource' , Const.ltsCh1 );


% Start measurement:
Result = calllib( sDLL , 'ADC_Start' );
% nCount is used to create a trigger time out for DLL's not supporting SetTriggerTimeOut. 
nCount = 0;
while ~calllib( sDLL , 'ADC_Ready' ) & nCount < 500
  nCount = nCount + 1;
  pause(.01); % This pause is essential for DLL's relying on a message loop!
end
if nCount >= 500
  calllib( sDLL , 'ADC_ForceTrig' );
  disp 'Time-out';
end


% Create array with a pointer to it:
darData = zeros( nRecordLength , 1 , 'double' );  
pData = libpointer('doublePtr', darData);

% Retrieve Volt doubles:
clear darVolt;
clear darRanges;
for i = 1:nChannelCount;
  [ Result , darVolt( : , i ) ] = calllib( sDLL , 'ADC_GetDataVoltCh' , i , pData );
  [ Result , darRanges( i ) ] = calllib( sDLL , 'GetSensitivity' , i , 0 );
end  
rRange = max( darRanges );

% Plot results:
figure(500);
plot( ( 1:nRecordLength ) / rSampleFrequency , darVolt );
axis( [ 0 (nRecordLength / rSampleFrequency) -rRange rRange ] );
xlabel( 'Time [s]' );
ylabel( 'Amplitude [V]' );

% Plot FFT:
figure(501);
FFT = abs( fft( darVolt ) ) * 2 / nRecordLength;
plot( ( 1:( nRecordLength / 2 ) ) * rSampleFrequency / nRecordLength , FFT( 1:( nRecordLength / 2 ) , : ) );
xlabel( 'Frequency [Hz]' );
ylabel( 'Amplitude [V]' );

